@@ -16,6 +16,9 @@
  * @endcopyright
  */
 package org.tigris.subversion.javahl;
+
+import java.io.OutputStream;
+
 /**
  * This class provides a threadsafe wrapped for SVNClient
  */
@@ -36,7 +39,10 @@ public class SVNClientSynchronized implements SVNClientInterface
      */
     public SVNClientSynchronized()
     {
-        worker = new SVNClient();
+        synchronized(clazz)
+        {
+            worker = new SVNClient();
+        }
     }
 
     /**
@@ -47,6 +53,48 @@ public class SVNClientSynchronized implements SVNClientInterface
         worker.dispose();
     }
 
+    /**
+     * @return Version information about the underlying native libraries.
+     */
+    public Version getVersion()
+    {
+        synchronized(clazz)
+        {
+            return worker.getVersion();
+        }
+    }
+
+    /**
+     * @return The name of the working copy's administrative
+     * directory, which is usually <code>.svn</code>.
+     * @see <a
+     * href="http://svn.collab.net/repos/svn/trunk/notes/asp-dot-net-hack.txt">Instructions</a>
+     * on changing this as a work-around for the behavior of ASP.Net
+     * on Windows.
+     * @since 1.3
+     */
+    public String getAdminDirectoryName()
+    {
+        synchronized(clazz)
+        {
+            return worker.getAdminDirectoryName();
+        }
+    }
+
+    /**
+     * @param name The name of the directory to compare.
+     * @return Whether <code>name</code> is that of a working copy
+     * administrative directory.
+     * @since 1.3
+     */
+    public boolean isAdminDirectory(String name)
+    {
+        synchronized(clazz)
+        {
+            return worker.isAdminDirectory(name);
+        }
+    }
+
     /**
      * Returns the last destination path submitted.
      * @deprecated
@@ -64,7 +112,7 @@ public class SVNClientSynchronized implements SVNClientInterface
      * List a directory or file of the working copy.
      *
      * @param path      Path to explore.
-     * @param descend   Recurse into subdirectories if existant.
+     * @param descend   Recurse into subdirectories if they exist.
      * @param onServer  Request status information from server.
      * @param getAll    get status for uninteristing files (unchanged).
      * @return Array of Status entries.
@@ -81,7 +129,7 @@ public class SVNClientSynchronized implements SVNClientInterface
      * List a directory or file of the working copy.
      *
      * @param path      Path to explore.
-     * @param descend   Recurse into subdirectories if existant.
+     * @param descend   Recurse into subdirectories if they exist.
      * @param onServer  Request status information from server.
      * @param getAll    get status for uninteristing files (unchanged).
      * @param noIgnore  get status for normaly ignored files and directories.
@@ -101,7 +149,7 @@ public class SVNClientSynchronized implements SVNClientInterface
      * List a directory or file of the working copy.
      *
      * @param path            Path to explore.
-     * @param descend         Recurse into subdirectories if existant.
+     * @param descend         Recurse into subdirectories if they exist.
      * @param onServer        Request status information from server.
      * @param getAll          get status for uninteristing files (unchanged).
      * @param noIgnore        get status for normaly ignored files and
@@ -280,8 +328,11 @@ public class SVNClientSynchronized implements SVNClientInterface
                                     boolean discoverPath, long limit)
             throws ClientException
     {
-        return worker.logMessages(path, revisionStart, revisionEnd,
-                stopOnCopy, discoverPath, limit);
+        synchronized (clazz)
+        {
+            return worker.logMessages(path, revisionStart, revisionEnd,
+                    stopOnCopy, discoverPath, limit);
+        }
     }
 
     /**
@@ -1170,6 +1221,27 @@ public class SVNClientSynchronized implements SVNClientInterface
         }
     }
 
+    /**
+     * Write the file's content to the specified output stream.
+     *
+     * @param path        the path of the file
+     * @param revision    the revision to retrieve
+     * @param pegRevision the revision at which to interpret the path
+     * @param the stream to write the file's content to
+     * @throws ClientException
+     */
+    public void streamFileContent(String path, Revision revision,
+                                  Revision pegRevision, int bufferSize,
+                                  OutputStream stream) 
+        throws ClientException
+    {
+        synchronized(clazz)
+        {
+            worker.streamFileContent(path, revision, pegRevision, bufferSize,
+                                     stream);
+        }
+    }
+
     /**
      * Rewrite the url's in the working copy
      * @param from      old url
