@@ -320,6 +320,9 @@ void svn_swig_py_svn_exception(svn_error_t *err)
 
   /* Finished with the exc_ob object. */
   Py_DECREF(exc_ob);
+
+  /* Consume the Subversion error. */
+  svn_error_clear(err);
 }
 
 
@@ -604,8 +607,6 @@ apr_hash_t *svn_swig_py_stringhash_from_dict(PyObject *dict,
       PyObject *value = PyDict_GetItem(dict, key);
       const char *propname = make_string_from_ob(key, pool);
       const char *propval = make_string_from_ob(value, pool);
-      Py_DECREF(key);
-      Py_DECREF(value);
       if (! (propname && propval))
         {
           PyErr_SetString(PyExc_TypeError, 
@@ -644,8 +645,6 @@ apr_hash_t *svn_swig_py_prophash_from_dict(PyObject *dict,
       PyObject *value = PyDict_GetItem(dict, key);
       const char *propname = make_string_from_ob(key, pool);
       svn_string_t *propval = make_svn_string_from_ob(value, pool);
-      Py_DECREF(key);
-      Py_DECREF(value);
       if (! (propname && propval))
         {
           PyErr_SetString(PyExc_TypeError, 
