@@ -1,7 +1,7 @@
 /**
  * @copyright
  * ====================================================================
- * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2006 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -21,7 +21,7 @@
  * This is an internalized library for performing contextual diffs
  * between sources of data.
  *
- * NOTE: this is different than Subversion's binary-diffing engine.
+ * @note This is different than Subversion's binary-diffing engine.
  * That API lives in @c svn_delta.h -- see the "text deltas" section.  A
  * "text delta" is way of representing precise binary diffs between
  * strings of data.  The Subversion client and server send text deltas
@@ -59,18 +59,18 @@ extern "C" {
 
 
 /**
- * @since New in 1.1.
- *
  * Get libsvn_diff version information.
+ *
+ * @since New in 1.1.
  */
-const svn_version_t *svn_diff_version (void);
+const svn_version_t *svn_diff_version(void);
 
 
 /* Diffs. */
 
 /** An opaque type that represents a difference between either two or
- * three datasources.   This object is returned by @c svn_diff_diff(),
- * @c svn_diff_diff3() and @c svn_diff_diff4, and consumed by a number of
+ * three datasources.   This object is returned by svn_diff_diff(),
+ * svn_diff_diff3() and svn_diff_diff4(), and consumed by a number of
  * other routines.
  */
 typedef struct svn_diff_t svn_diff_t;
@@ -110,7 +110,10 @@ typedef struct svn_diff_fns_t
   svn_error_t *(*datasource_close)(void *diff_baton,
                                    svn_diff_datasource_e datasource);
 
-  /** Get the next "token" from the datasource of type @a datasource. */
+  /** Get the next "token" from the datasource of type @a datasource.
+   *  Return a "token" in @a *token.   Return a hash of "token" in @a *hash.
+   *  Leave @a token and @a hash untouched when the datasource is exhausted.
+   */
   svn_error_t *(*datasource_get_next_token)(apr_uint32_t *hash, void **token,
                                             void *diff_baton,
                                             svn_diff_datasource_e datasource);
@@ -193,7 +196,7 @@ svn_diff_contains_diffs(svn_diff_t *diff);
  * Differences, similarities, and conflicts are described by lining up
  * "ranges" of data.
  *  
- * Note: these callbacks describe data ranges in units of "tokens".
+ * @note These callbacks describe data ranges in units of "tokens".
  * A "token" is whatever you've defined it to be in your datasource
  * @c svn_diff_fns_t vtable.
  */
@@ -299,11 +302,89 @@ svn_diff_output(svn_diff_t *diff,
 
 /* Diffs on files */
 
+/** To what extent whitespace should be ignored when comparing lines.
+ *
+ * @since New in 1.4.
+ */
+typedef enum svn_diff_file_ignore_space_t
+{
+  /** Ignore no whitespace. */
+  svn_diff_file_ignore_space_none,
+
+  /** Ignore changes in sequences of whitespace characters, treating each
+   * sequence of whitespace characters as a single space. */
+  svn_diff_file_ignore_space_change,
+
+  /** Ignore all whitespace characters. */
+  svn_diff_file_ignore_space_all
+} svn_diff_file_ignore_space_t;
+
+/** Options to control the behaviour of the file diff routines.
+ *
+ * @since New in 1.4.
+ *
+ * @note This structure may be extended in the future, so to preserve binary
+ * compatibility, users must not allocate structs of this type themselves.
+ * @see svn_diff_file_options_create().
+ */
+typedef struct svn_diff_file_options_t
+{
+  /** To what extent whitespace should be ignored when comparing lines.
+   * The default is @c svn_diff_file_ignore_space_none. */
+  svn_diff_file_ignore_space_t ignore_space;
+  /** Whether to treat all end-of-line markers the same when comparing lines.
+   * The default is @c FALSE. */
+  svn_boolean_t ignore_eol_style;
+} svn_diff_file_options_t;
+
+/** Allocate a @c svn_diff_file_options_t structure in @a pool, initializing
+ * it with default values.
+ *
+ * @since New in 1.4.
+ */
+svn_diff_file_options_t *
+svn_diff_file_options_create(apr_pool_t *pool);
+
+/**
+ * Parse @a args, an array of <tt>const char *</tt> command line switches
+ * and adjust @a options accordingly.  @a options is assumed to be initialized
+ * with default values.  @a pool is used for temporary allocation.
+ *
+ * @since New in 1.4.
+ *
+ * The following options are supported:
+ * - --ignore-space-change, -b
+ * - --ignore-all-space, -w
+ * - --ignore-eol-style
+ * - --unified, -u (for compatibility, does nothing).
+ */
+svn_error_t *
+svn_diff_file_options_parse(svn_diff_file_options_t *options,
+                            const apr_array_header_t *args,
+                            apr_pool_t *pool);
+                            
+
 /** A convenience function to produce a diff between two files.
+ *
+ * @since New in 1.4.
  *
  * Return a diff object in @a *diff (allocated from @a pool) that represents
  * the difference between an @a original file and @a modified file.  
  * (The file arguments must be full paths to the files.)
+ *
+ * Compare lines according to the relevant fields of @a options.
+ */
+svn_error_t *
+svn_diff_file_diff_2(svn_diff_t **diff,
+                     const char *original,
+                     const char *modified,
+                     const svn_diff_file_options_t *options,
+                     apr_pool_t *pool);
+
+/** Similar to svn_file_diff_2(), but with @a options set to a struct with
+ * default options.
+ *
+ * @deprecated Provided for backwards compatibility with the 1.3 API.
  */
 svn_error_t *
 svn_diff_file_diff(svn_diff_t **diff,
@@ -311,12 +392,28 @@ svn_diff_file_diff(svn_diff_t **diff,
                    const char *modified,
                    apr_pool_t *pool);
 
-
 /** A convenience function to produce a diff between three files.
+ *
+ * @since New in 1.4.
  *
  * Return a diff object in @a *diff (allocated from @a pool) that represents
  * the difference between an @a original file, @a modified file, and @a latest 
  * file. (The file arguments must be full paths to the files.)
+ *
+ * Compare lines according to the relevant fields of @a options.
+ */
+svn_error_t *
+svn_diff_file_diff3_2(svn_diff_t **diff,
+                      const char *original,
+                      const char *modified,
+                      const char *latest,
+                      const svn_diff_file_options_t *options,
+                      apr_pool_t *pool);
+
+/** Similar to svn_diff_file_diff3_2(), but with @a options set to a struct
+ * with default options.
+ *
+ * @deprecated Provided for backwards compatibility with the 1.3 API.
  */
 svn_error_t *
 svn_diff_file_diff3(svn_diff_t **diff,
@@ -326,10 +423,28 @@ svn_diff_file_diff3(svn_diff_t **diff,
                     apr_pool_t *pool);
 
 /** A convenience function to produce a diff between four files.
+ *
+ * @since New in 1.4.
  *
  * Return a diff object in @a *diff (allocated from @a pool) that represents
  * the difference between an @a original file, @a modified file, @a latest
  * and @a ancestor file. (The file arguments must be full paths to the files.)
+ *
+ * Compare lines according to the relevant fields of @a options.
+ */
+svn_error_t *
+svn_diff_file_diff4_2(svn_diff_t **diff,
+                      const char *original,
+                      const char *modified,
+                      const char *latest,
+                      const char *ancestor,
+                      const svn_diff_file_options_t *options,
+                      apr_pool_t *pool);
+
+/** Simliar to svn_file_diff4_2(), but with @a options set to a struct with
+ * default options.
+ *
+ * @deprecated Provided for backwards compatibility with the 1.3 API.
  */
 svn_error_t *
 svn_diff_file_diff4(svn_diff_t **diff,
@@ -340,13 +455,31 @@ svn_diff_file_diff4(svn_diff_t **diff,
                     apr_pool_t *pool);
 
 /** A convenience function to produce unified diff output from the
- * diff generated by @c svn_diff_file.
+ * diff generated by svn_diff_file_diff().
+ *
+ * @since New in 1.3.
  *
  * Output a @a diff between @a original_path and @a modified_path in unified
  * context diff format to @a output_file.  Optionally supply @a original_header
  * and/or @a modified_header to be displayed in the header of the output.
  * If @a original_header or @a modified_header is @c NULL, a default header 
- * will be displayed, consisting of path and last modified time.
+ * will be displayed, consisting of path and last modified time.  Output
+ * all headers and markers in @a header_encoding.
+ */
+svn_error_t *
+svn_diff_file_output_unified2(svn_stream_t *output_stream,
+                              svn_diff_t *diff,
+                              const char *original_path,
+                              const char *modified_path,
+                              const char *original_header,
+                              const char *modified_header,
+                              const char *header_encoding,
+                              apr_pool_t *pool);
+
+/** Similar to svn_diff_file_output_unified2(), but with @a header_encoding
+ * set to @c APR_LOCALE_CHARSET.
+ *
+ * @deprecated Provided for backward compatibility with the 1.2 API.
  */
 svn_error_t *
 svn_diff_file_output_unified(svn_stream_t *output_stream,
@@ -359,7 +492,7 @@ svn_diff_file_output_unified(svn_stream_t *output_stream,
 
 
 /** A convenience function to produce diff3 output from the
- * diff generated by @c svn_diff3_file.
+ * diff generated by svn_diff_file_diff3().
  *
  * Output a @a diff between @a original_path, @a modified_path and
  * @a latest_path in merged format to @a output_file.  Optionally supply
