@@ -424,8 +424,8 @@ svn_log_changed_path_dup(const svn_log_changed_path_t *changed_path,
  * think they are.  Any of @a author, @a date, or @a message may be @c NULL.
  *
  * If @a date is neither null nor the empty string, it was generated by
- * svn_time_to_string() and can be converted to @c apr_time_t with
- * svn_time_from_string().
+ * svn_time_to_cstring() and can be converted to @c apr_time_t with
+ * svn_time_from_cstring().
  *
  * If @a changed_paths is non-@c NULL, then it contains as keys every path
  * committed in @a revision; the values are (@c svn_log_changed_path_t *) 
@@ -438,17 +438,19 @@ svn_log_changed_path_dup(const svn_log_changed_path_t *changed_path,
  * they're not, it might be nice to change apr_hash_first() so
  * read-only uses of hashes can be protected via the type system.
  *
- * Use @a pool for all allocation.  (If the caller is iterating over log
- * messages, invoking this receiver on each, we recommend the standard
- * pool loop recipe: create a subpool, pass it as @a pool to each call,
- * clear it after each iteration, destroy it after the loop is done.)
+ * Use @a pool for temporary allocation.  If the caller is iterating
+ * over log messages, invoking this receiver on each, we recommend the
+ * standard pool loop recipe: create a subpool, pass it as @a pool to
+ * each call, clear it after each iteration, destroy it after the loop
+ * is done.  (For allocation that must last beyond the lifetime of a
+ * given receiver call, use a pool in @a baton.)
  */
 typedef svn_error_t *(*svn_log_message_receiver_t)
   (void *baton,
    apr_hash_t *changed_paths,
    svn_revnum_t revision,
    const char *author,
-   const char *date,  /* use svn_time_from_string() if need apr_time_t */
+   const char *date,  /* use svn_time_from_cstring() if need apr_time_t */
    const char *message,
    apr_pool_t *pool);
 
