@@ -44,7 +44,7 @@ extern "C" {
 
 typedef struct svn_txdelta__ops_baton_t {
   int num_ops;                  /* current number of ops */
-  int src_ops;                  /* current number of source copy ope */
+  int src_ops;                  /* current number of source copy ops */
   int ops_size;                 /* number of ops allocated */
   svn_txdelta_op_t *ops;        /* the operations */
 
@@ -56,53 +56,33 @@ typedef struct svn_txdelta__ops_baton_t {
    OPCODE is svn_delta_new, bytes from NEW_DATA are copied into the window
    data and OFFSET is ignored.  Otherwise NEW_DATA is ignored. All
    allocations are performed in POOL. */
-void svn_txdelta__insert_op (svn_txdelta__ops_baton_t *build_baton,
-                             enum svn_delta_action opcode,
-                             apr_size_t offset,
-                             apr_size_t length,
-                             const char *new_data,
-                             apr_pool_t *pool);
+void svn_txdelta__insert_op(svn_txdelta__ops_baton_t *build_baton,
+                            enum svn_delta_action opcode,
+                            apr_size_t offset,
+                            apr_size_t length,
+                            const char *new_data,
+                            apr_pool_t *pool);
 
 
 /* Allocate a delta window from POOL. */
 svn_txdelta_window_t *
-svn_txdelta__make_window (const svn_txdelta__ops_baton_t *build_baton,
-                          apr_pool_t *pool);
+svn_txdelta__make_window(const svn_txdelta__ops_baton_t *build_baton,
+                         apr_pool_t *pool);
 
 /* Create vdelta window data. Allocate temporary data from POOL. */
-void svn_txdelta__vdelta (svn_txdelta__ops_baton_t *build_baton,
-                          const char *start,
-                          apr_size_t source_len,
-                          apr_size_t target_len,
-                          apr_pool_t *pool);
+void svn_txdelta__vdelta(svn_txdelta__ops_baton_t *build_baton,
+                         const char *start,
+                         apr_size_t source_len,
+                         apr_size_t target_len,
+                         apr_pool_t *pool);
 
 
 /* Create xdelta window data. Allocate temporary data from POOL. */
-void svn_txdelta__xdelta (svn_txdelta__ops_baton_t *build_baton,
-                          const char *start,
-                          apr_size_t source_len,
-                          apr_size_t target_len,
-                          apr_pool_t *pool);
-
-/* Compose two delta windows, yielding a third, allocated from POOL. */
-svn_txdelta_window_t *
-svn_txdelta__compose_windows (const svn_txdelta_window_t *window_A,
-                              const svn_txdelta_window_t *window_B,
-                              apr_pool_t *pool);
-
-
-/* Apply the instructions from WINDOW to a source view SBUF to produce
-   a target view TBUF.  SBUF is assumed to have WINDOW->sview_len
-   bytes of data and TBUF is assumed to have room for TLEN bytes of
-   output.  TLEN may be more than WINDOW->tview_len, so return the
-   actual number of bytes written.  SBUF is not touched and may be
-   NULL if WINDOW contains no source-copy operations. This is purely a
-   memory operation; nothing can go wrong as long as we have a valid
-   window.  */
-void
-svn_txdelta__apply_instructions (svn_txdelta_window_t *window,
-                                 const char *sbuf, char *tbuf,
-                                 apr_size_t *tlen);
+void svn_txdelta__xdelta(svn_txdelta__ops_baton_t *build_baton,
+                         const char *start,
+                         apr_size_t source_len,
+                         apr_size_t target_len,
+                         apr_pool_t *pool);
 
 
 #ifdef __cplusplus
