@@ -58,6 +58,8 @@ static const svn_ra_dav__xml_elm_t merge_elements[] =
   { "DAV:", "collection", ELEM_collection, 0 },
   { "DAV:", "baseline", ELEM_baseline, 0 },
   { "DAV:", "version-name", ELEM_version_name, SVN_RA_DAV__XML_CDATA },
+  { SVN_XML_NAMESPACE, "post-commit-err",
+    ELEM_post_commit_err, SVN_RA_DAV__XML_CDATA },
   { "DAV:", "creationdate", ELEM_creationdate, SVN_RA_DAV__XML_CDATA },
   { "DAV:", "creator-displayname", ELEM_creator_displayname,
     SVN_RA_DAV__XML_CDATA },
@@ -104,6 +106,8 @@ typedef struct {
   svn_stringbuf_t *committed_date; /* DAV:creationdate for this resource */
   svn_stringbuf_t *last_author;    /* DAV:creator-displayname for this
                                       resource */
+  svn_stringbuf_t *post_commit_err;/* SVN_XML_NAMESPACE:post-commit hook's
+                                      stderr */
 
   /* We only invoke set_prop() on targets listed in valid_targets.
      Some entities (such as directories that have had changes
@@ -126,33 +130,32 @@ static void add_ignored(merge_ctx_t *mc, const char *cdata)
 }
 
 
-static svn_boolean_t okay_to_bump_path (const char *path,
-                                        apr_hash_t *valid_targets,
-                                        apr_pool_t *pool)
+static svn_boolean_t okay_to_bump_path(const char *path,
+                                       apr_hash_t *valid_targets,
+                                       apr_pool_t *pool)
 {
   svn_stringbuf_t *parent_path;
   enum svn_recurse_kind r;
 
   /* Easy check:  if path itself is in the hash, then it's legit. */
-  if (apr_hash_get (valid_targets, path, APR_HASH_KEY_STRING))
+  if (apr_hash_get(valid_targets, path, APR_HASH_KEY_STRING))
     return TRUE;
-
   /* Otherwise, this path is bumpable IFF one of its parents is in the
      hash and marked with a 'recursion' flag. */
-  parent_path = svn_stringbuf_create (path, pool);
+  parent_path = svn_stringbuf_create(path, pool);
   
   do {
     apr_size_t len = parent_path->len;
-    svn_path_remove_component (parent_path);
+    svn_path_remove_component(parent_path);
     if (len == parent_path->len)
       break;
-    r = (enum svn_recurse_kind) apr_hash_get (valid_targets,
-                                              parent_path->data,
-                                              APR_HASH_KEY_STRING);
+    r = (enum svn_recurse_kind) apr_hash_get(valid_targets,
+                                             parent_path->data,
+                                             APR_HASH_KEY_STRING);
     if (r == svn_recursive)
       return TRUE;
 
-  } while (! svn_path_is_empty (parent_path->data));
+  } while (! svn_path_is_empty(parent_path->data));
 
   /* Default answer: if we get here, don't allow the bumping. */
   return FALSE;
@@ -179,7 +182,7 @@ static svn_error_t *bump_resource(merge_ctx_t *mc,
      committed target.  The commit-tracking editor built this list for
      us, and took care not to include directories unless they were
      directly committed (i.e., received a property change). */
-  if (! okay_to_bump_path (path, mc->valid_targets, pool))
+  if (! okay_to_bump_path(path, mc->valid_targets, pool))
     return SVN_NO_ERROR;
 
   /* Okay, NOW set the new version url. */
@@ -189,9 +192,9 @@ static svn_error_t *bump_resource(merge_ctx_t *mc,
     vsn_url_str.data = vsn_url;
     vsn_url_str.len = strlen(vsn_url);
 
-    SVN_ERR( (*mc->push_prop)(mc->cb_baton, path,
-                              SVN_RA_DAV__LP_VSN_URL, &vsn_url_str,
-                              pool) );
+    SVN_ERR((*mc->push_prop)(mc->cb_baton, path,
+                             SVN_RA_DAV__LP_VSN_URL, &vsn_url_str,
+                             pool));
   }
 
   return SVN_NO_ERROR;
@@ -269,7 +272,7 @@ static svn_error_t * handle_resource(merge_ctx_t *mc,
     relative = mc->href->data + mc->base_len + 1;
 
   /* bump the resource */
-  relative = svn_path_uri_decode (relative, pool);
+  relative = svn_path_uri_decode(relative, pool);
   return bump_resource(mc, relative, mc->vsn_url->data, pool);
 }
 
@@ -341,6 +344,7 @@ static int validate_element(void *userdata, svn_ra_dav__xml_elmid parent,
           || child == ELEM_version_name
           || child == ELEM_creationdate
           || child == ELEM_creator_displayname
+          || child == ELEM_post_commit_err
           /* other props */)
         return SVN_RA_DAV__XML_VALID;
       else
@@ -509,7 +513,7 @@ static int end_element(void *userdata, const svn_ra_dav__xml_elm_t *elm,
             else
               svn_error_clear(err);
           }
-        svn_pool_clear (mc->scratchpool);
+        svn_pool_clear(mc->scratchpool);
       }
       break;
 
@@ -524,6 +528,10 @@ static int end_element(void *userdata, const svn_ra_dav__xml_elm_t *elm,
       svn_stringbuf_set(mc->vsn_name, cdata);
       break;
 
+    case ELEM_post_commit_err:
+      svn_stringbuf_set(mc->post_commit_err, cdata);
+      break;
+
     case ELEM_creationdate:
       svn_stringbuf_set(mc->committed_date, cdata);
       break;
@@ -656,6 +664,7 @@ svn_error_t * svn_ra_dav__merge_activity(
     svn_revnum_t *new_rev,
     const char **committed_date,
     const char **committed_author,
+    const char **post_commit_err,
     svn_ra_dav__session_t *ras,
     const char *repos_url,
     const char *activity_url,
@@ -672,7 +681,7 @@ svn_error_t * svn_ra_dav__merge_activity(
   svn_error_t *err;
 
   mc.pool = pool;
-  mc.scratchpool = svn_pool_create (pool);
+  mc.scratchpool = svn_pool_create(pool);
   mc.base_href = repos_url;
   mc.base_len = strlen(repos_url);
   mc.rev = SVN_INVALID_REVNUM;
@@ -686,6 +695,8 @@ svn_error_t * svn_ra_dav__merge_activity(
   mc.vsn_url = MAKE_BUFFER(pool);
   mc.committed_date = MAKE_BUFFER(pool);
   mc.last_author = MAKE_BUFFER(pool);
+  if (post_commit_err)
+    mc.post_commit_err = MAKE_BUFFER(pool);
 
   if (disable_merge_response 
       || (! keep_locks))
@@ -700,15 +711,15 @@ svn_error_t * svn_ra_dav__merge_activity(
       
       if (! extra_headers)
         extra_headers = apr_hash_make(pool);
-      apr_hash_set (extra_headers, SVN_DAV_OPTIONS_HEADER, APR_HASH_KEY_STRING,
-                    value);
+      apr_hash_set(extra_headers, SVN_DAV_OPTIONS_HEADER, APR_HASH_KEY_STRING,
+                   value);
     }
 
   /* Need to marshal the whole [path->token] hash to the server as
      a string within the body of the MERGE request. */
   if ((lock_tokens != NULL)
       && (apr_hash_count(lock_tokens) > 0))
-    SVN_ERR( svn_ra_dav__assemble_locktoken_body(&lockbuf, lock_tokens, pool) );
+    SVN_ERR(svn_ra_dav__assemble_locktoken_body(&lockbuf, lock_tokens, pool));
 
   body = apr_psprintf(pool,
                       "<?xml version=\"1.0\" encoding=\"utf-8\"?>"
@@ -749,6 +760,9 @@ svn_error_t * svn_ra_dav__merge_activity(
   if (committed_author)
     *committed_author = mc.last_author->len 
                         ? apr_pstrdup(pool, mc.last_author->data) : NULL;
+  if (post_commit_err)
+    *post_commit_err = mc.post_commit_err->len
+                        ? apr_pstrdup(pool, mc.post_commit_err->data) : NULL;
 
   svn_pool_destroy(mc.scratchpool);
 
