@@ -34,6 +34,7 @@
 #include "svn_delta.h"
 #include "svn_ra.h"
 #include "svn_dav.h"
+#include "svn_private_config.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -143,6 +144,9 @@ struct lock_request_baton
   /* If <D:error> is returned, here's where the parsed result goes. */
   svn_error_t *err;
 
+  /* The neon request being executed */
+  ne_request *request;
+
   /* A place for allocating fields in this structure. */
   apr_pool_t *pool;
 };
@@ -150,14 +154,14 @@ struct lock_request_baton
 
 typedef struct {
   apr_pool_t *pool;
-  const char *url;                      /* original, unparsed session url */
+  svn_stringbuf_t *url;                 /* original, unparsed session url */
   ne_uri root;                          /* parsed version of above */
   const char *repos_root;               /* URL for repository root */
 
   ne_session *sess;                     /* HTTP session to server */
   ne_session *sess2;
 
-  const svn_ra_callbacks_t *callbacks;  /* callbacks to get auth data */
+  const svn_ra_callbacks2_t *callbacks; /* callbacks to get auth data */
   void *callback_baton;
  
   svn_auth_iterstate_t *auth_iterstate; /* state of authentication retries */
@@ -169,6 +173,8 @@ typedef struct {
   
   struct lock_request_baton *lrb;       /* used by lock/unlock */
 
+  struct copy_baton *cb;                /* used by COPY */
+
 } svn_ra_dav__session_t;
 
 
@@ -190,141 +196,132 @@ svn_error_t *svn_ra_dav__get_latest_revnum(svn_ra_session_t *session,
                                            svn_revnum_t *latest_revnum,
                                            apr_pool_t *pool);
 
-svn_error_t *svn_ra_dav__get_dated_revision (svn_ra_session_t *session,
-                                             svn_revnum_t *revision,
-                                             apr_time_t timestamp,
-                                             apr_pool_t *pool);
+svn_error_t *svn_ra_dav__get_dated_revision(svn_ra_session_t *session,
+                                            svn_revnum_t *revision,
+                                            apr_time_t timestamp,
+                                            apr_pool_t *pool);
 
-svn_error_t *svn_ra_dav__change_rev_prop (svn_ra_session_t *session,
-                                          svn_revnum_t rev,
-                                          const char *name,
-                                          const svn_string_t *value,
-                                          apr_pool_t *pool);
+svn_error_t *svn_ra_dav__change_rev_prop(svn_ra_session_t *session,
+                                         svn_revnum_t rev,
+                                         const char *name,
+                                         const svn_string_t *value,
+                                         apr_pool_t *pool);
 
-svn_error_t *svn_ra_dav__rev_proplist (svn_ra_session_t *session,
-                                       svn_revnum_t rev,
-                                       apr_hash_t **props,
-                                       apr_pool_t *pool);
+svn_error_t *svn_ra_dav__rev_proplist(svn_ra_session_t *session,
+                                      svn_revnum_t rev,
+                                      apr_hash_t **props,
+                                      apr_pool_t *pool);
 
-svn_error_t *svn_ra_dav__rev_prop (svn_ra_session_t *session,
-                                   svn_revnum_t rev,
-                                   const char *name,
-                                   svn_string_t **value,
+svn_error_t *svn_ra_dav__rev_prop(svn_ra_session_t *session,
+                                  svn_revnum_t rev,
+                                  const char *name,
+                                  svn_string_t **value,
+                                  apr_pool_t *pool);
+
+svn_error_t * svn_ra_dav__get_commit_editor(svn_ra_session_t *session,
+                                            const svn_delta_editor_t **editor,
+                                            void **edit_baton,
+                                            const char *log_msg,
+                                            svn_commit_callback2_t callback,
+                                            void *callback_baton,
+                                            apr_hash_t *lock_tokens,
+                                            svn_boolean_t keep_locks,
+                                            apr_pool_t *pool);
+
+svn_error_t * svn_ra_dav__get_file(svn_ra_session_t *session,
+                                   const char *path,
+                                   svn_revnum_t revision,
+                                   svn_stream_t *stream,
+                                   svn_revnum_t *fetched_rev,
+                                   apr_hash_t **props,
                                    apr_pool_t *pool);
 
-svn_error_t * svn_ra_dav__get_commit_editor(
-  svn_ra_session_t *session,
-  const svn_delta_editor_t **editor,
-  void **edit_baton,
-  const char *log_msg,
-  svn_commit_callback_t callback,
-  void *callback_baton,
-  apr_hash_t *lock_tokens,
-  svn_boolean_t keep_locks,
-  apr_pool_t *pool);
+svn_error_t *svn_ra_dav__get_dir(svn_ra_session_t *session,
+                                 apr_hash_t **dirents,
+                                 svn_revnum_t *fetched_rev,
+                                 apr_hash_t **props,
+                                 const char *path,
+                                 svn_revnum_t revision,
+                                 apr_uint32_t dirent_fields,
+                                 apr_pool_t *pool);
 
-svn_error_t * svn_ra_dav__get_file(
-  svn_ra_session_t *session,
-  const char *path,
-  svn_revnum_t revision,
-  svn_stream_t *stream,
-  svn_revnum_t *fetched_rev,
-  apr_hash_t **props,
-  apr_pool_t *pool);
+svn_error_t * svn_ra_dav__abort_commit(void *session_baton,
+                                       void *edit_baton);
 
-svn_error_t *svn_ra_dav__get_dir(
-  svn_ra_session_t *session,
-  const char *path,
-  svn_revnum_t revision,
-  apr_hash_t **dirents,
-  svn_revnum_t *fetched_rev,
-  apr_hash_t **props,
-  apr_pool_t *pool);
+svn_error_t * svn_ra_dav__do_update(svn_ra_session_t *session,
+                                    const svn_ra_reporter2_t **reporter,
+                                    void **report_baton,
+                                    svn_revnum_t revision_to_update_to,
+                                    const char *update_target,
+                                    svn_boolean_t recurse,
+                                    const svn_delta_editor_t *wc_update,
+                                    void *wc_update_baton,
+                                    apr_pool_t *pool);
 
-svn_error_t * svn_ra_dav__abort_commit(
- void *session_baton,
- void *edit_baton);
+svn_error_t * svn_ra_dav__do_status(svn_ra_session_t *session,
+                                    const svn_ra_reporter2_t **reporter,
+                                    void **report_baton,
+                                    const char *status_target,
+                                    svn_revnum_t revision,
+                                    svn_boolean_t recurse,
+                                    const svn_delta_editor_t *wc_status,
+                                    void *wc_status_baton,
+                                    apr_pool_t *pool);
 
-svn_error_t * svn_ra_dav__do_update(
-  svn_ra_session_t *session,
-  const svn_ra_reporter2_t **reporter,
-  void **report_baton,
-  svn_revnum_t revision_to_update_to,
-  const char *update_target,
-  svn_boolean_t recurse,
-  const svn_delta_editor_t *wc_update,
-  void *wc_update_baton,
-  apr_pool_t *pool);
+svn_error_t * svn_ra_dav__do_switch(svn_ra_session_t *session,
+                                    const svn_ra_reporter2_t **reporter,
+                                    void **report_baton,
+                                    svn_revnum_t revision_to_update_to,
+                                    const char *update_target,
+                                    svn_boolean_t recurse,
+                                    const char *switch_url,
+                                    const svn_delta_editor_t *wc_update,
+                                    void *wc_update_baton,
+                                    apr_pool_t *pool);
 
-svn_error_t * svn_ra_dav__do_status(
-  svn_ra_session_t *session,
-  const svn_ra_reporter2_t **reporter,
-  void **report_baton,
-  const char *status_target,
-  svn_revnum_t revision,
-  svn_boolean_t recurse,
-  const svn_delta_editor_t *wc_status,
-  void *wc_status_baton,
-  apr_pool_t *pool);
+svn_error_t * svn_ra_dav__do_diff(svn_ra_session_t *session,
+                                  const svn_ra_reporter2_t **reporter,
+                                  void **report_baton,
+                                  svn_revnum_t revision,
+                                  const char *diff_target,
+                                  svn_boolean_t recurse,
+                                  svn_boolean_t ignore_ancestry,
+                                  svn_boolean_t text_deltas,
+                                  const char *versus_url,
+                                  const svn_delta_editor_t *wc_diff,
+                                  void *wc_diff_baton,
+                                  apr_pool_t *pool);
 
-svn_error_t * svn_ra_dav__do_switch(
-  svn_ra_session_t *session,
-  const svn_ra_reporter2_t **reporter,
-  void **report_baton,
-  svn_revnum_t revision_to_update_to,
-  const char *update_target,
-  svn_boolean_t recurse,
-  const char *switch_url,
-  const svn_delta_editor_t *wc_update,
-  void *wc_update_baton,
-  apr_pool_t *pool);
+svn_error_t * svn_ra_dav__get_log(svn_ra_session_t *session,
+                                  const apr_array_header_t *paths,
+                                  svn_revnum_t start,
+                                  svn_revnum_t end,
+                                  int limit,
+                                  svn_boolean_t discover_changed_paths,
+                                  svn_boolean_t strict_node_history,
+                                  svn_log_message_receiver_t receiver,
+                                  void *receiver_baton,
+                                  apr_pool_t *pool);
 
-svn_error_t * svn_ra_dav__do_diff(
-  svn_ra_session_t *session,
-  const svn_ra_reporter2_t **reporter,
-  void **report_baton,
-  svn_revnum_t revision,
-  const char *diff_target,
-  svn_boolean_t recurse,
-  svn_boolean_t ignore_ancestry,
-  const char *versus_url,
-  const svn_delta_editor_t *wc_diff,
-  void *wc_diff_baton,
-  apr_pool_t *pool);
+svn_error_t *svn_ra_dav__do_check_path(svn_ra_session_t *session,
+                                       const char *path,
+                                       svn_revnum_t revision,
+                                       svn_node_kind_t *kind,
+                                       apr_pool_t *pool);
 
-svn_error_t * svn_ra_dav__get_log(
-  svn_ra_session_t *session,
-  const apr_array_header_t *paths,
-  svn_revnum_t start,
-  svn_revnum_t end,
-  int limit,
-  svn_boolean_t discover_changed_paths,
-  svn_boolean_t strict_node_history,
-  svn_log_message_receiver_t receiver,
-  void *receiver_baton,
-  apr_pool_t *pool);
+svn_error_t *svn_ra_dav__do_stat(svn_ra_session_t *session,
+                                 const char *path,
+                                 svn_revnum_t revision,
+                                 svn_dirent_t **dirent,
+                                 apr_pool_t *pool);
 
-svn_error_t *svn_ra_dav__do_check_path(
-  svn_ra_session_t *session,
-  const char *path,
-  svn_revnum_t revision,
-  svn_node_kind_t *kind,
-  apr_pool_t *pool);
-
-svn_error_t *svn_ra_dav__do_stat(
-  svn_ra_session_t *session,
-  const char *path,
-  svn_revnum_t revision,
-  svn_dirent_t **dirent,
-  apr_pool_t *pool);
-
-svn_error_t *svn_ra_dav__get_file_revs (svn_ra_session_t *session,
-                                        const char *path,
-                                        svn_revnum_t start,
-                                        svn_revnum_t end,
-                                        svn_ra_file_rev_handler_t handler,
-                                        void *handler_baton,
-                                        apr_pool_t *pool);
+svn_error_t *svn_ra_dav__get_file_revs(svn_ra_session_t *session,
+                                       const char *path,
+                                       svn_revnum_t start,
+                                       svn_revnum_t end,
+                                       svn_ra_file_rev_handler_t handler,
+                                       void *handler_baton,
+                                       apr_pool_t *pool);
 
 
 /*
@@ -365,6 +362,7 @@ svn_error_t *svn_ra_dav__get_file_revs (svn_ra_session_t *session,
 
 #define SVN_RA_DAV__PROP_REPOSITORY_UUID SVN_DAV_PROP_NS_DAV "repository-uuid"
 
+#define SVN_RA_DAV__PROP_DEADPROP_COUNT SVN_DAV_PROP_NS_DAV "deadprop-count"
 
 typedef struct {
   /* what is the URL for this resource */
@@ -504,12 +502,12 @@ svn_error_t *svn_ra_dav__get_vcc(const char **vcc,
    property names) and PROP_DELETES (an array of property names to
    delete).  Send any extra request headers in EXTRA_HEADERS. Use POOL
    for all allocations.  */
-svn_error_t *svn_ra_dav__do_proppatch (svn_ra_dav__session_t *ras,
-                                       const char *url,
-                                       apr_hash_t *prop_changes,
-                                       apr_array_header_t *prop_deletes,
-                                       apr_hash_t *extra_headers,
-                                       apr_pool_t *pool);
+svn_error_t *svn_ra_dav__do_proppatch(svn_ra_dav__session_t *ras,
+                                      const char *url,
+                                      apr_hash_t *prop_changes,
+                                      apr_array_header_t *prop_deletes,
+                                      apr_hash_t *extra_headers,
+                                      apr_pool_t *pool);
 
 extern const ne_propname svn_ra_dav__vcc_prop;
 extern const ne_propname svn_ra_dav__checked_in_prop;
@@ -518,11 +516,11 @@ extern const ne_propname svn_ra_dav__checked_in_prop;
 
 
 /* send an OPTIONS request to fetch the activity-collection-set */
-svn_error_t * svn_ra_dav__get_activity_collection(
-  const svn_string_t **activity_coll,
-  svn_ra_dav__session_t *ras,
-  const char *url,
-  apr_pool_t *pool);
+svn_error_t * svn_ra_dav__get_activity_collection
+  (const svn_string_t **activity_coll,
+   svn_ra_dav__session_t *ras,
+   const char *url,
+   apr_pool_t *pool);
 
 
 /* Call ne_set_request_body_pdovider on REQ with a provider function
@@ -561,7 +559,11 @@ svn_ra_dav__lookup_xml_elem(const svn_ra_dav__xml_elm_t *table,
  * inserted as extra headers in the request.  Can be NULL.
  *
  * STATUS_CODE is an optional 'out' parameter; if non-NULL, then set
- * *STATUS_CODE to the http status code returned by the server.
+ * *STATUS_CODE to the http status code returned by the server.  This
+ * can be set to a useful value even when the function returns an error
+ * however it is not always set when an error is returned.  So any caller
+ * wishing to check *STATUS_CODE when an error has been returned must
+ * initialise *STATUS_CODE before calling the function.
  *
  * If SPOOL_RESPONSE is set, the request response will be cached to
  * disk in a tmpfile (in full), then read back and parsed.
@@ -574,8 +576,8 @@ svn_ra_dav__parsed_request(ne_session *sess,
                            const char *url,
                            const char *body,
                            apr_file_t *body_file,
-                           void set_parser (ne_xml_parser *parser,
-                                            void *baton),
+                           void set_parser(ne_xml_parser *parser,
+                                           void *baton),
                            ne_xml_startelm_cb *startelm_cb,
                            ne_xml_cdata_cb *cdata_cb,
                            ne_xml_endelm_cb *endelm_cb,
@@ -599,8 +601,8 @@ svn_ra_dav__parsed_request_compat(ne_session *sess,
                                   const char *url,
                                   const char *body,
                                   apr_file_t *body_file,
-                                  void set_parser (ne_xml_parser *parser,
-                                                   void *baton),
+                                  void set_parser(ne_xml_parser *parser,
+                                                  void *baton),
                                   const svn_ra_dav__xml_elm_t *elements, 
                                   svn_ra_dav__xml_validate_cb validate_cb,
                                   svn_ra_dav__xml_startelm_cb startelm_cb, 
@@ -654,6 +656,7 @@ enum {
   ELEM_updated_set,
   ELEM_vcc,
   ELEM_version_name,
+  ELEM_post_commit_err,
   ELEM_error,
 
   /* SVN elements */
@@ -700,22 +703,30 @@ enum {
   ELEM_lock_owner,
   ELEM_lock_comment,
   ELEM_lock_creationdate,
-  ELEM_lock_expirationdate
+  ELEM_lock_expirationdate,
+  ELEM_editor_report,
+  ELEM_open_root,
+  ELEM_apply_textdelta,
+  ELEM_change_file_prop,
+  ELEM_change_dir_prop,
+  ELEM_close_file,
+  ELEM_close_directory,
+  ELEM_deadprop_count
 };
 
 /* ### docco */
-svn_error_t * svn_ra_dav__merge_activity(
-    svn_revnum_t *new_rev,
-    const char **committed_date,
-    const char **committed_author,
-    svn_ra_dav__session_t *ras,
-    const char *repos_url,
-    const char *activity_url,
-    apr_hash_t *valid_targets,
-    apr_hash_t *lock_tokens,
-    svn_boolean_t keep_locks,
-    svn_boolean_t disable_merge_response,
-    apr_pool_t *pool);
+svn_error_t * svn_ra_dav__merge_activity(svn_revnum_t *new_rev,
+                                         const char **committed_date,
+                                         const char **committed_author,
+                                         const char **post_commit_err,
+                                         svn_ra_dav__session_t *ras,
+                                         const char *repos_url,
+                                         const char *activity_url,
+                                         apr_hash_t *valid_targets,
+                                         apr_hash_t *lock_tokens,
+                                         svn_boolean_t keep_locks,
+                                         svn_boolean_t disable_merge_response,
+                                         apr_pool_t *pool);
 
 
 /* Make a buffer for repeated use with svn_stringbuf_set().
@@ -729,9 +740,10 @@ void svn_ra_dav__copy_href(svn_stringbuf_t *dst, const char *src);
 
 
 /* If RAS contains authentication info, attempt to store it via client
-   callbacks.  */
+   callbacks and using POOL for temporary allocations.  */
 svn_error_t *
-svn_ra_dav__maybe_store_auth_info (svn_ra_dav__session_t *ras);
+svn_ra_dav__maybe_store_auth_info(svn_ra_dav__session_t *ras,
+                                  apr_pool_t *pool);
 
 
 /* Like svn_ra_dav__maybe_store_auth_info(), but conditional on ERR.
@@ -742,7 +754,8 @@ svn_ra_dav__maybe_store_auth_info (svn_ra_dav__session_t *ras);
    store auth info, else return SVN_NO_ERROR. */
 svn_error_t *
 svn_ra_dav__maybe_store_auth_info_after_result(svn_error_t *err,
-                                               svn_ra_dav__session_t *ras);
+                                               svn_ra_dav__session_t *ras,
+                                               apr_pool_t *pool);
 
 
 /* Create an error object for an error from neon in the given session,
@@ -754,6 +767,17 @@ svn_error_t *svn_ra_dav__convert_error(ne_session *sess,
                                        apr_pool_t *pool);
 
 
+/* Callback to get data from a Neon request after it has been sent.
+
+   REQUEST is the request, DISPATCH_RETURN_VAL is the value that
+   ne_request_dispatch(REQUEST) returned to the caller.
+
+   USERDATA is a closure baton. */
+typedef svn_error_t *
+svn_ra_dav__request_interrogator(ne_request *request,
+                                 int dispatch_return_val,
+                                 void *userdata);
+
 /* Given a neon REQUEST and SESSION, run the request; if CODE_P is
    non-null, return the http status code in *CODE_P.  Return any
    resulting error (from neon, a <D:error> body response, or any
@@ -768,6 +792,16 @@ svn_error_t *svn_ra_dav__convert_error(ne_session *sess,
    specified (e.g. as 200); use 0 for OKAY_2 if a second result code is
    not allowed.
 
+   #ifdef SVN_NEON_0_25
+
+      If INTERROGATOR is non-NULL, invoke it with the Neon request,
+      the dispatch result, and INTERROGATOR_BATON.  This is done
+      regardless of whether the request appears successful or not.  If
+      the interrogator has an error result, return that error
+      immediately, after freeing the request.
+
+   #endif // SVN_NEON_0_25
+
    ### not super sure on this "okay" stuff, but it means that the request
    ### dispatching code can generate much better errors than the callers
    ### when something goes wrong. if we need more than two, then we could
@@ -782,6 +816,10 @@ svn_ra_dav__request_dispatch(int *code_p,
                              const char *url,
                              int okay_1,
                              int okay_2,
+#ifdef SVN_NEON_0_25
+                             svn_ra_dav__request_interrogator interrogator,
+                             void *interrogator_baton,
+#endif /* SVN_NEON_0_25 */
                              apr_pool_t *pool);
 
 
@@ -799,12 +837,12 @@ svn_ra_dav__add_error_handler(ne_request *request,
 /*
  * Implements the get_locations RA layer function. */
 svn_error_t *
-svn_ra_dav__get_locations (svn_ra_session_t *session,
-                           apr_hash_t **locations,
-                           const char *path,
-                           svn_revnum_t peg_revision,
-                           apr_array_header_t *location_revisions,
-                           apr_pool_t *pool);
+svn_ra_dav__get_locations(svn_ra_session_t *session,
+                          apr_hash_t **locations,
+                          const char *path,
+                          svn_revnum_t peg_revision,
+                          apr_array_header_t *location_revisions,
+                          apr_pool_t *pool);
 
 
 /*
@@ -823,6 +861,14 @@ svn_ra_dav__get_lock(svn_ra_session_t *session,
                      const char *path,
                      apr_pool_t *pool);
 
+svn_error_t *
+svn_ra_dav__replay(svn_ra_session_t *session,
+                   svn_revnum_t revision,
+                   svn_revnum_t low_water_mark,
+                   svn_boolean_t send_deltas,
+                   const svn_delta_editor_t *editor,
+                   void *edit_baton,
+                   apr_pool_t *pool);
 
 /* Helper function.  Loop over LOCK_TOKENS and assemble all keys and
    values into a stringbuf allocated in POOL.  The string will be of
