@@ -501,7 +501,8 @@ static svn_error_t *link_path(svn_ra_svn_conn_t *conn, apr_pool_t *pool,
   path = svn_path_canonicalize(path, pool);
   url = svn_path_uri_decode(svn_path_canonicalize(url, pool), pool);
   if (!b->err)
-    b->err = get_fs_path(b->repos_url, url, &fs_path, pool);
+    b->err = get_fs_path(svn_path_uri_decode(b->repos_url, pool), url,
+                         &fs_path, pool);
   if (!b->err)
     b->err = svn_repos_link_path2(b->report_baton, path, fs_path, rev,
                                   start_empty, lock_token, pool);
@@ -708,7 +709,8 @@ static svn_error_t *reparent(svn_ra_svn_conn_t *conn, apr_pool_t *pool,
   SVN_ERR(svn_ra_svn_parse_tuple(params, pool, "c", &url));
   url = svn_path_uri_decode(svn_path_canonicalize(url, pool), pool);
   SVN_ERR(trivial_auth_request(conn, pool, b));
-  SVN_CMD_ERR(get_fs_path(b->repos_url, url, &fs_path, pool));
+  SVN_CMD_ERR(get_fs_path(svn_path_uri_decode(b->repos_url, pool), url,
+                          &fs_path, pool));
   svn_stringbuf_set(b->fs_path, fs_path);
   SVN_ERR(svn_ra_svn_write_cmd_response(conn, pool, ""));
   return SVN_NO_ERROR;
@@ -833,7 +835,7 @@ static svn_error_t *add_lock_tokens(svn_ra_svn_conn_t *conn,
 
   for (i = 0; i < lock_tokens->nelts; ++i)
     {
-      const char *token;
+      const char *path, *token, *full_path;
       svn_ra_svn_item_t *path_item, *token_item;
       svn_ra_svn_item_t *item = &APR_ARRAY_IDX(lock_tokens, i,
                                                svn_ra_svn_item_t);
@@ -851,8 +853,13 @@ static svn_error_t *add_lock_tokens(svn_ra_svn_conn_t *conn,
         return svn_error_create(SVN_ERR_RA_SVN_MALFORMED_DATA, NULL,
                                 "Lock token isn't a string");
 
+      path = path_item->u.string->data;
+      full_path = svn_path_join(sb->fs_path->data,
+                                svn_path_canonicalize(path, pool),
+                                pool);
+
       if (! lookup_access(pool, sb, svn_authz_write,
-                          path_item->u.string->data, TRUE))
+                          full_path, TRUE))
         return svn_error_create(SVN_ERR_RA_NOT_AUTHORIZED,
                                 NULL, NULL);
 
