@@ -28,19 +28,6 @@ Skip = svntest.testcase.Skip
 XFail = svntest.testcase.XFail
 Item = wc.StateItem
 
-#----------------------------------------------------------------------
-
-def expect_extra_files(node, extra_files):
-  """singleton handler for expected singletons"""
-
-  for pattern in extra_files:
-    mo = re.match(pattern, node.name)
-    if mo:
-      extra_files.pop(extra_files.index(pattern))
-      return
-  print "Found unexpected object:", node.name
-  raise svntest.main.SVNTreeUnequal
-
 ######################################################################
 # Tests
 #
@@ -569,13 +556,13 @@ Original appended text for rho
                  'rho.*\.r1', 'rho.*\.r2', 'rho.*\.mine',]
   
   # Do the update and check the results in three ways.
-  # All "extra" files are passed to expect_extra_files().
+  # All "extra" files are passed to detect_conflict_files().
   svntest.actions.run_and_verify_update(wc_backup,
                                         expected_output,
                                         expected_disk,
                                         expected_status,
                                         None,
-                                        expect_extra_files,
+                                        svntest.tree.detect_conflict_files,
                                         extra_files)
   
   # verify that the extra_files list is now empty.
@@ -1762,7 +1749,21 @@ def checkout_peg_rev(sbox):
                                           checkout_target, 
                                           expected_output,
                                           expected_wc)
-  
+
+# Issue #2612.
+def ls_space_in_repo_name(sbox):
+  'basic ls of repos with space in name'
+
+  sbox.build(name = "repo with spaces")
+  wc_dir = sbox.wc_dir
+
+  svntest.actions.run_and_verify_svn('ls the root of the repository',
+                                     ['A/\n', 'iota\n'],
+                                     [], 'ls',
+                                     '--username', svntest.main.wc_author,
+                                     '--password', svntest.main.wc_passwd,
+                                     sbox.repo_url)
+
 ########################################################################
 # Run the tests
 
@@ -1801,6 +1802,7 @@ test_list = [ None,
               cat_added_PREV,
               checkout_creates_intermediate_folders,
               checkout_peg_rev,
+              ls_space_in_repo_name,
               ### todo: more tests needed:
               ### test "svn rm http://some_url"
               ### not sure this file is the right place, though.
