@@ -20,21 +20,18 @@ def usage():
 
 def run(files):
 
-  svn.core.apr_initialize()
-  pool = svn.core.svn_pool_create(None)
-
   for f in files:
     dirpath = fullpath = os.path.abspath(f)
     if not os.path.isdir(dirpath):
       dirpath = os.path.dirname(dirpath)
   
-    adm_baton = svn.wc.adm_open(None, dirpath, False, True, pool)
+    adm_baton = svn.wc.adm_open(None, dirpath, False, True)
 
     try:
-      entry = svn.wc.entry(fullpath, adm_baton, 0, pool)
+      entry = svn.wc.entry(fullpath, adm_baton, 0)
 
       if svn.wc.text_modified_p(fullpath, FORCE_COMPARISON,
-				adm_baton, pool):
+				adm_baton):
         print "M      %s" % f
       else:
         print "       %s" % f
@@ -43,9 +40,6 @@ def run(files):
 
     svn.wc.adm_close(adm_baton)
 
-  svn.core.svn_pool_destroy(pool)
-  svn.core.apr_terminate()        
-
 if __name__ == '__main__':
   run(sys.argv[1:])
     
