@@ -11,17 +11,17 @@ import sys
 import svn.wc
 import svn.core
 
-def main(pool, files):
+def main(files):
   for f in files:
     dirpath = fullpath = os.path.abspath(f)
     if not os.path.isdir(dirpath):
       dirpath = os.path.dirname(dirpath)
-    adm_baton = svn.wc.adm_open(None, dirpath, 1, 1, pool)
+    adm_baton = svn.wc.adm_open(None, dirpath, 1, 1)
     try:
-      entry = svn.wc.entry(fullpath, adm_baton, 0, pool)
+      entry = svn.wc.entry(fullpath, adm_baton, 0)
       print entry.url
     finally:
       svn.wc.adm_close(adm_baton)
 
 if __name__ == '__main__':
-  svn.core.run_app(main, sys.argv[1:])
+  main(sys.argv[1:])
