@@ -9,7 +9,7 @@ BUILD_TYPE="$1"
 # Compute local vars
 LOG_FILE="$LOG_FILE_PREFIX.$BUILD_TYPE"
 BUILD="`$GUESS` $BUILD_TYPE"
-REV="`$SVN st -v $SVN_REPO/README | $CUT -c 12-17 | $SED -e 's/^ *//'`"
+REV="`$SVN st -v $SVN_SOURCE/README | $CUT -c 12-17 | $SED -e 's/^ *//'`"
 
 # Initialize the log file
 echo "BUILD: $REVPREFIX$REV on $BUILD" >> $LOG_FILE
